//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*
-------------------------------------------------------------------------------
   Title      : Data bus intreface for ZBT Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : CTRL_BUS_INTERFACE.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : defines.v
-------------------------------------------------------------------------------
   Used by    : ZBT_CONTROL.v
-------------------------------------------------------------------------------
   Description: This module defines the data bus interface for the ZBT RAM 
		    controller. 
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
*/

`define ADDR_BITS 19
`define DATA_BITS 32



module DATA_BUS_INTERFACE(
read_data, 
//ram_data_bus, 
ram_data_bus_T,
ram_data_bus_O,
ram_data_bus_I,
write_data,tristate_control,clock)/* synthesis syn_useioff = 1*/;

// Interface to DATABITS_INOUT
output   [(`DATA_BITS-1):0]     read_data;
input    [(`DATA_BITS-1):0]     write_data;
input    [(`DATA_BITS-1):0]     tristate_control;
input                           clock;

// Interface to ZBT SRAM
//inout    [(`DATA_BITS-1):0]    ram_data_bus;
output     [(`DATA_BITS-1):0]    ram_data_bus_T;
output     [(`DATA_BITS-1):0]    ram_data_bus_O;
input      [(`DATA_BITS-1):0]    ram_data_bus_I;

reg [(`DATA_BITS-1):0] latched_write_data;
reg [(`DATA_BITS-1):0] latched_tristate_control;
//tri [(`DATA_BITS-1):0] ram_data_bus;

always @ (posedge clock) begin
	latched_write_data 	 <= write_data;
	latched_tristate_control <= tristate_control;
end

assign ram_data_bus_T    = latched_tristate_control;
assign ram_data_bus_O    = latched_write_data;
assign read_data         = ram_data_bus_I;

// instantiate the IO buffers
//IOBUF_F_12 I_data_buf0	( .O(read_data[0]), .IO(ram_data_bus[0]),  .I(latched_write_data[0]),  .T(latched_tristate_control[0]));
//IOBUF_F_12 I_data_buf1	( .O(read_data[1]), .IO(ram_data_bus[1]),  .I(latched_write_data[1]),  .T(latched_tristate_control[1]));
//IOBUF_F_12 I_data_buf2	( .O(read_data[2]), .IO(ram_data_bus[2]),  .I(latched_write_data[2]),  .T(latched_tristate_control[2]));
//IOBUF_F_12 I_data_buf3	( .O(read_data[3]), .IO(ram_data_bus[3]),  .I(latched_write_data[3]),  .T(latched_tristate_control[3]));
//IOBUF_F_12 I_data_buf4	( .O(read_data[4]), .IO(ram_data_bus[4]),  .I(latched_write_data[4]),  .T(latched_tristate_control[4]));
//IOBUF_F_12 I_data_buf5	( .O(read_data[5]), .IO(ram_data_bus[5]),  .I(latched_write_data[5]),  .T(latched_tristate_control[5]));
//IOBUF_F_12 I_data_buf6	( .O(read_data[6]), .IO(ram_data_bus[6]),  .I(latched_write_data[6]),  .T(latched_tristate_control[6]));
//IOBUF_F_12 I_data_buf7	( .O(read_data[7]), .IO(ram_data_bus[7]),  .I(latched_write_data[7]),  .T(latched_tristate_control[7]));
//IOBUF_F_12 I_data_buf8	( .O(read_data[8]), .IO(ram_data_bus[8]),  .I(latched_write_data[8]),  .T(latched_tristate_control[8]));
//IOBUF_F_12 I_data_buf9	( .O(read_data[9]), .IO(ram_data_bus[9]),  .I(latched_write_data[9]),  .T(latched_tristate_control[9]));
//IOBUF_F_12 I_data_buf10	( .O(read_data[10]),.IO(ram_data_bus[10]), .I(latched_write_data[10]), .T(latched_tristate_control[10]));
//IOBUF_F_12 I_data_buf11	( .O(read_data[11]),.IO(ram_data_bus[11]), .I(latched_write_data[11]), .T(latched_tristate_control[11]));
//IOBUF_F_12 I_data_buf12	( .O(read_data[12]),.IO(ram_data_bus[12]), .I(latched_write_data[12]), .T(latched_tristate_control[12]));
//IOBUF_F_12 I_data_buf13	( .O(read_data[13]),.IO(ram_data_bus[13]), .I(latched_write_data[13]), .T(latched_tristate_control[13]));
//IOBUF_F_12 I_data_buf14	( .O(read_data[14]),.IO(ram_data_bus[14]), .I(latched_write_data[14]), .T(latched_tristate_control[14]));
//IOBUF_F_12 I_data_buf15	( .O(read_data[15]),.IO(ram_data_bus[15]), .I(latched_write_data[15]), .T(latched_tristate_control[15]));
//IOBUF_F_12 I_data_buf16	( .O(read_data[16]),.IO(ram_data_bus[16]), .I(latched_write_data[16]), .T(latched_tristate_control[16]));
//IOBUF_F_12 I_data_buf17	( .O(read_data[17]),.IO(ram_data_bus[17]), .I(latched_write_data[17]), .T(latched_tristate_control[17]));
//IOBUF_F_12 I_data_buf18	( .O(read_data[18]),.IO(ram_data_bus[18]), .I(latched_write_data[18]), .T(latched_tristate_control[18]));
//IOBUF_F_12 I_data_buf19	( .O(read_data[19]),.IO(ram_data_bus[19]), .I(latched_write_data[19]), .T(latched_tristate_control[19]));
//IOBUF_F_12 I_data_buf20	( .O(read_data[20]),.IO(ram_data_bus[20]), .I(latched_write_data[20]), .T(latched_tristate_control[20]));
//IOBUF_F_12 I_data_buf21	( .O(read_data[21]),.IO(ram_data_bus[21]), .I(latched_write_data[21]), .T(latched_tristate_control[21]));
//IOBUF_F_12 I_data_buf22	( .O(read_data[22]),.IO(ram_data_bus[22]), .I(latched_write_data[22]), .T(latched_tristate_control[22]));
//IOBUF_F_12 I_data_buf23	( .O(read_data[23]),.IO(ram_data_bus[23]), .I(latched_write_data[23]), .T(latched_tristate_control[23]));
//IOBUF_F_12 I_data_buf24	( .O(read_data[24]),.IO(ram_data_bus[24]), .I(latched_write_data[24]), .T(latched_tristate_control[24]));
//IOBUF_F_12 I_data_buf25	( .O(read_data[25]),.IO(ram_data_bus[25]), .I(latched_write_data[25]), .T(latched_tristate_control[25]));
//IOBUF_F_12 I_data_buf26	( .O(read_data[26]),.IO(ram_data_bus[26]), .I(latched_write_data[26]), .T(latched_tristate_control[26]));
//IOBUF_F_12 I_data_buf27	( .O(read_data[27]),.IO(ram_data_bus[27]), .I(latched_write_data[27]), .T(latched_tristate_control[27]));
//IOBUF_F_12 I_data_buf28	( .O(read_data[28]),.IO(ram_data_bus[28]), .I(latched_write_data[28]), .T(latched_tristate_control[28]));
//IOBUF_F_12 I_data_buf29	( .O(read_data[29]),.IO(ram_data_bus[29]), .I(latched_write_data[29]), .T(latched_tristate_control[29]));
//IOBUF_F_12 I_data_buf30	( .O(read_data[30]),.IO(ram_data_bus[30]), .I(latched_write_data[30]), .T(latched_tristate_control[30]));
//IOBUF_F_12 I_data_buf31	( .O(read_data[31]),.IO(ram_data_bus[31]), .I(latched_write_data[31]), .T(latched_tristate_control[31]));
//IOBUF_F_12 I_data_buf32	( .O(read_data[32]),.IO(ram_data_bus[32]), .I(latched_write_data[32]), .T(latched_tristate_control[32]));
//IOBUF_F_12 I_data_buf33	( .O(read_data[33]),.IO(ram_data_bus[33]), .I(latched_write_data[33]), .T(latched_tristate_control[33]));
//IOBUF_F_12 I_data_buf34	( .O(read_data[34]),.IO(ram_data_bus[34]), .I(latched_write_data[34]), .T(latched_tristate_control[34]));
//IOBUF_F_12 I_data_buf35	( .O(read_data[35]),.IO(ram_data_bus[35]), .I(latched_write_data[35]), .T(latched_tristate_control[35]));
  
endmodule // DATABITS_INOUT




