//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*

-------------------------------------------------------------------------------
   Title      : Data Bus Pipeline for ZBT Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : PIPELINES.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : defines.v
-------------------------------------------------------------------------------
   Used by    : ZBT_CONTROL.v
-------------------------------------------------------------------------------
   Description: This module data pipelines for ZBT data WRITES and a latch
		    for the data READs
-------------------------------------------------------------------------------
  
-------------------------------------------------------------------------------
*/


`define ADDR_BITS 19
`define DATA_BITS 32



module PIPELINES (
user_read_data, 
write_data,
tristate_control, 
user_write_data, 
user_read_writez,
read_data, 
clock
);

// interfaces to user logic
input					clock;
input [(`DATA_BITS-1):0]	user_write_data;
input 		  		user_read_writez;
output [(`DATA_BITS-1):0] 	user_read_data;

// interfaces to DATA_BUS_INTERFACE
output [(`DATA_BITS-1):0] 	write_data;
input  [(`DATA_BITS-1):0]  	read_data;
output [(`DATA_BITS-1):0] 	tristate_control;



reg [(`DATA_BITS-1):0]  user_read_data;
reg [(`DATA_BITS-1):0]  write_data;
reg [(`DATA_BITS-1):0] write_data_pipe1;

wire [3:0] 		  	read_write_pipe1;	// instantiated FD
reg  [3:0] 		  	read_write_pipe2;
reg  [3:0] 		  	read_write_pipe3;

wire [(`DATA_BITS-1):0] tristate_control; // instantiated FD


// instantiate FDs to reduce the fanout on the read writes and tristates
FD FD_RW0  (.Q(read_write_pipe1[0]),   .C(clock), .D(user_read_writez));
FD FD_RW1  (.Q(read_write_pipe1[1]),   .C(clock), .D(user_read_writez));
FD FD_RW2  (.Q(read_write_pipe1[2]),   .C(clock), .D(user_read_writez));
FD FD_RW3  (.Q(read_write_pipe1[3]),   .C(clock), .D(user_read_writez));

FD FD_TS0  (.Q(tristate_control[0]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS1  (.Q(tristate_control[1]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS2  (.Q(tristate_control[2]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS3  (.Q(tristate_control[3]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS4  (.Q(tristate_control[4]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS5  (.Q(tristate_control[5]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS6  (.Q(tristate_control[6]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS7  (.Q(tristate_control[7]),  .C(clock), .D(read_write_pipe1[0]));
FD FD_TS8  (.Q(tristate_control[8]),  .C(clock), .D(read_write_pipe1[0]));

FD FD_TS9  (.Q(tristate_control[9]),  .C(clock), .D(read_write_pipe1[1]));
FD FD_TS10 (.Q(tristate_control[10]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS11 (.Q(tristate_control[11]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS12 (.Q(tristate_control[12]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS13 (.Q(tristate_control[13]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS14 (.Q(tristate_control[14]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS15 (.Q(tristate_control[15]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS16 (.Q(tristate_control[16]), .C(clock), .D(read_write_pipe1[1]));
FD FD_TS17 (.Q(tristate_control[17]), .C(clock), .D(read_write_pipe1[1]));

FD FD_TS18 (.Q(tristate_control[18]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS19 (.Q(tristate_control[19]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS20 (.Q(tristate_control[20]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS21 (.Q(tristate_control[21]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS22 (.Q(tristate_control[22]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS23 (.Q(tristate_control[23]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS24 (.Q(tristate_control[24]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS25 (.Q(tristate_control[25]), .C(clock), .D(read_write_pipe1[2]));
FD FD_TS26 (.Q(tristate_control[26]), .C(clock), .D(read_write_pipe1[2]));

FD FD_TS27 (.Q(tristate_control[27]), .C(clock), .D(read_write_pipe1[3]));
FD FD_TS28 (.Q(tristate_control[28]), .C(clock), .D(read_write_pipe1[3]));
FD FD_TS29 (.Q(tristate_control[29]), .C(clock), .D(read_write_pipe1[3]));
FD FD_TS30 (.Q(tristate_control[30]), .C(clock), .D(read_write_pipe1[3]));
FD FD_TS31 (.Q(tristate_control[31]), .C(clock), .D(read_write_pipe1[3]));
//FD FD_TS32 (.Q(tristate_control[32]), .C(clock), .D(read_write_pipe1[3]));
//FD FD_TS33 (.Q(tristate_control[33]), .C(clock), .D(read_write_pipe1[3]));
//FD FD_TS34 (.Q(tristate_control[34]), .C(clock), .D(read_write_pipe1[3]));
//FD FD_TS35 (.Q(tristate_control[35]), .C(clock), .D(read_write_pipe1[3]));

// add two pipeline stages to data being written to the ZBT SRAM
always @ (posedge clock) begin
	write_data_pipe1 	<= user_write_data;
	write_data 		<= write_data_pipe1;
	end

// there are four levels in the read_write pipe line to compensate
// for the two levels in the ram and the two level in the controller
always @ (posedge clock) begin
	read_write_pipe2[3:0] <= read_write_pipe1[3:0];
	read_write_pipe3[3:0] <= read_write_pipe2[3:0];
	end

// register the data being read back from the ZBT SRAM
/*
always @ (posedge clock) begin
	if (read_write_pipe3[0]) user_read_data[8:0]   <= read_data[8:0];
	if (read_write_pipe3[1]) user_read_data[17:9]  <= read_data[17:9];
	if (read_write_pipe3[2]) user_read_data[26:18] <= read_data[26:18];
	if (read_write_pipe3[3]) user_read_data[35:27] <= read_data[35:27];
end
*/
always @ (posedge clock) begin
	if (read_write_pipe3[0]) user_read_data[7:0]   <= read_data[7:0];
	if (read_write_pipe3[1]) user_read_data[15:8]  <= read_data[15:8];
	if (read_write_pipe3[2]) user_read_data[23:16] <= read_data[23:16];
	if (read_write_pipe3[3]) user_read_data[31:24] <= read_data[31:24];
end
endmodule // PIPELINES


